/******************************************************************************
 *
 * comment.c - NTFS file attributes comments. Part of the DirTools project.
 *
 * $Revision: 1.1.1.1 $
 *
 * Copyright (c) 2001-2004 Olivier Castan
 *
 * This program/include file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program/include file is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (in the main directory of the Linux-NTFS
 * distribution in the file COPYING); if not, write to the Free Software
 * Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * format : indent -br -ce -cdw -nut -i5
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdint.h>

#include "dt_comment.h"
#include "dt_error.h"

/**********************************************************************
 * DumpBits
 *
 * - [in] fOutput : pointer to file descriptor
 *
 * - [in] bCompact : if TRUE prints output using parseable format
 *
 *********************************************************************/
void
DumpBits (FILE * fOutput, uint32_t dwBits, TYPE_COMMENT_BITS cbRef[],
          int bCompact)
{
     TYPE_COMMENT_BITS *pTable = NULL;
     int bFirst = TRUE;
     uint32_t dwResult = 0;

     if (cbRef == NULL) {
          ALARM (TA_PROG_ERR, "no reference");
     } else {
          for (pTable = cbRef; pTable->dwBits; pTable++) {
               if (((dwBits & pTable->dwBits) == pTable->dwBits) &&
                   ((pTable->dwBits & dwResult) != pTable->dwBits)) {
                    fprintf (fOutput, ((bCompact || bFirst) ? "%s" : ", %s"),
                             (bCompact ? pTable->pComment[0] : pTable->
                              pComment[1]));
                    bFirst = FALSE;
                    dwResult |= pTable->dwBits;
               }
          }
          fprintf (fOutput, (bCompact ? "|" : "\n"));
     }
}
