/******************************************************************************
 *
 * dt_error.h - errors handling routine. Part of the DirTools project.
 *
 * $Revision: 1.2 $
 *
 * Copyright (c) 2001-2004 Olivier Castan
 *
 * This program/include file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program/include file is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (in the main directory of the Linux-NTFS
 * distribution in the file COPYING); if not, write to the Free Software
 * Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * format : indent -br -ce -cdw -nut -i5
 *
 *****************************************************************************/

#ifndef _DT_ERROR_H_
#define _DT_ERROR_H_ 1

#include <stdio.h>
#include <stdarg.h>
#include <stdint.h>
#ifdef _WIN32
#include <windows.h>
#else
#include "wincompat.h"
#endif /* _WIN32 */

#define OK_ERROR  0
#define FATAL_ERROR  1
#define SIMPLE_ERROR  2

#define ALARM(level, msg, ...) Alarm(level, "%s %s() l.%d: "msg, \
                                     __FILE__, __func__, __LINE__, \
                                     ##__VA_ARGS__)

#define TA_SYSTEM 1
#define TA_PROGRAM 2
#define TA_WARNING 16
#define TA_ERROR 32
#define TA_FATAL 64

typedef enum
{
     TA_SYS_WARN = TA_SYSTEM | TA_WARNING,
     TA_SYS_ERR = TA_SYSTEM | TA_ERROR,
     TA_SYS_FATAL = TA_SYSTEM | TA_FATAL,
     TA_PROG_WARN = TA_PROGRAM | TA_WARNING,
     TA_PROG_ERR = TA_PROGRAM | TA_ERROR,
     TA_PROG_FATAL = TA_PROGRAM | TA_FATAL
} TYPE_ALARM;

extern FILE *FErrorOutput;

void Alarm (const TYPE_ALARM, const char *, ...);

#endif /* _DT_ERROR_H_ */
