/******************************************************************************
 *
 * structure.h - Structures for NTFS I/O. Part of the DirTools project.
 *
 * $Revision: 1.2 $
 *
 * Copyright (c) 2001-2004 Olivier Castan
 *
 * This program/include file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program/include file is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (in the main directory of the Linux-NTFS
 * distribution in the file COPYING); if not, write to the Free Software
 * Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * format : indent -br -ce -cdw -nut -i5
 *
 *****************************************************************************/

#ifndef _STRUCTURE_H_
#define _STRUCTURE_H_ 1

#include <stdlib.h>
#include <stdint.h>
#ifdef _WIN32
#include <windows.h>
#else
#include "wincompat.h"
#endif /* _WIN32 */

typedef struct _NTFS_MFT_ENTRY *PNTFS_MFT_ENTRY;
typedef struct _NTFS_ATTRIBUTE *PNTFS_ATTRIBUTE;

typedef struct _NTFS_SEC_INFO
{
     uint32_t dwSecId;
     uint32_t dwSize;
     uint64_t ulSDSOffset;
} NTFS_SEC_INFO, *PNTFS_SEC_INFO;

typedef struct _NTFS_SEC_LIST
{
     uint32_t dwListSize;
     uint32_t dwFirst;
     uint32_t dwMaxSize;
     PNTFS_SEC_INFO pSecInfo;
} NTFS_SEC_LIST;

#define CHUNK_CLUSTER_SIZE 16

typedef struct _NTFS_VOL_INFO
{
#ifdef _WIN32
     char tDrive;               /* Drive letter */
     HANDLE hVolume;
#else
     char *file;
     int hVolume;
     uint64_t ulSkip;
#endif                          /* _WIN32 */
     /* Volume data */
     uint32_t dwSectorSize;
     uint32_t dwClusterSize;
     uint64_t ulClusterNumber;
     uint64_t ulMftStart;
     uint64_t ulMftMirrStart;
     uint32_t dwMftRecordClusterSize;
     uint32_t dwMftRecordSize;
     uint32_t dwIndexRecordClusterSize;
     /* Values from $AttrDef record */
     uint32_t dwAtStdInfo;
     uint32_t dwAtAttrList;
     uint32_t dwAtFileName;
     uint32_t dwAtObjectID;
     uint32_t dwAtSecurity;
     uint32_t dwAtVolName;
     uint32_t dwAtVolInfo;
     uint32_t dwAtData;
     uint32_t dwAtIndexRoot;
     uint32_t dwAtIndexAlloc;
     uint32_t dwAtBitmap;
     uint32_t dwAtReparsePoint;
     uint32_t dwAtEaInf;
     uint32_t dwAtEa;
     uint32_t dwAtPropSet;
     uint32_t dwAtLogStream;
     uint16_t wMajorVersion;    /* NTFS major version i.e. 1 for NT4,
                                 * 3 for 2000, XP */
     uint32_t dwChunkSize;
     PNTFS_MFT_ENTRY pMft;
     PNTFS_MFT_ENTRY pSecure;
     PNTFS_ATTRIBUTE pMftData;
     PNTFS_ATTRIBUTE pSecureData;
     NTFS_SEC_LIST secList;
} NTFS_VOL_INFO, *PNTFS_VOL_INFO;

#define NTFS_VIRTUAL_LCN ((uint64_t) -1)
typedef struct _NTFS_RUNLIST
{
/*  uint64_t  ulFirstVcn; */
     uint64_t ulLcn;
     uint64_t ulLength;
} NTFS_RUNLIST, *PNTFS_RUNLIST;

#define ATTRIBUTE_MAX_NAME_LENGTH 256   /* Length coded on 1 byte + '\0' */

typedef struct _NTFS_ATTRIBUTE
{
     uint32_t dwType;
     int bResident;
     uint16_t wNameLength;
     wchar_t wcName[ATTRIBUTE_MAX_NAME_LENGTH];
     uint16_t wCompressed;
     uint64_t ulStreamSize;     /* Real size resident or not */
     /* Attribut rsident */
     unsigned char *pStream;
     /* Attribut non rsident */
     uint64_t ulAllocated;
     uint64_t ulCompressed;
     uint16_t wCompEngine;
     uint64_t ulRunlistSize;
     uint64_t ulVcnsNumber;
     uint64_t ulLastRunLcn;
     PNTFS_RUNLIST pRunlist;
} NTFS_ATTRIBUTE;

typedef struct _NTFS_MFT_ENTRY
{
     PNTFS_VOL_INFO pVolume;
     uint64_t ulNumber;
     uint16_t wSequence;
     uint16_t wRefCount;
     uint16_t wFlags;
     uint16_t wAttrCount;
     int bDirectory;
     PNTFS_ATTRIBUTE pAttr;
     int bSecIdFound;
     uint32_t dwSecId;
} NTFS_MFT_ENTRY;

#endif /* _STRUCTURE_H_  */
